import unittest
from src.part_library_gen.symbol_generator import svg_exporter
from src.part_library_gen.generators.components.symbol import Symbol
from src.part_library_gen.generators.components.part_number import PartNumber
from src.part_library_gen.generators.components.designator import Designator


class TestSVGExport(unittest.TestCase):
    def test_svg_export(self):

        symbol = Symbol(PartNumber("*", 10, 10),
                        Designator("*", 0, 0))

        svg = svg_exporter(symbol)
        self.assertIsNotNone(svg)



class Line:
    def __init__(self, x1, y1, x2, y2):
        self.x1 = x1
        self.y1 = y1
        self.x2 = x2
        self.y2 = y2

    def to_dict(self):
        return {
            self.x1: self.x1,
            self.y1: self.y1,
            self.x2: self.x2,
            self.y2: self.y2
        }



class Symbol:
    def __init__(self, part_number, designator):
        self.part_number = part_number
        self.designator = designator
        self.pins = []
        self.body = []
        self.width = 0
        self.height = 0

    def add_pin(self, pin):
        self.pins.append(pin)

    def add_body(self, element):
        self.body.append(element)

    def to_dict(self):
        return {
            'part_number': self.part_number.to_dict(),
            'designator': self.designator.to_dict(),
            'pins': [x.to_dict() for x in self.pins],
            'body': [x.to_dict() for x in self.body],
            'width': self.width,
            'height': self.height
        }

    def form_dict(self, data):
        self.part_number.from_dict(data['part_number'])
        self.designator.from_dict(data['designator'])
        self.width = data['width']
        self.height = data['height']



import drawsvg as svg
from drawsvg import Circle

from .pin import generate_symbol_pin
from ...generators.components.rectangle import Rectangle
from ...generators.components.line import Line
from ...generators.components.lines import Lines


def export(symbol, filename=None):
    d = svg.Drawing(symbol.width, symbol.height, origin='center')

    for element in symbol.body:
        if isinstance(element, Rectangle):
            d.append(svg.Rectangle(element.x,
                                   element.y,
                                   element.width,
                                   element.height,
                                   stroke_width=3,
                                   fill='yellow',
                                   stroke='black'))
        if isinstance(element, Lines):
            points = [element.x, element.y]
            for point in element.points:
                points += list(point)
            d.append(svg.Lines(*points,
                               stroke_width=4,
                               fill='yellow',
                               stroke='black'))
        if isinstance(element, Line):
            d.append(svg.Line(element.x1, element.y1, element.x2, element.y2, stroke_width=4, stroke='black'))

    for pin in symbol.pins:
        d.append(generate_symbol_pin(pin))

    d.append(svg.Text(symbol.designator.designator,
                      40,
                      symbol.designator.x,
                      symbol.designator.y))

    d.append(svg.Text(symbol.part_number.text,
                      40,
                      symbol.part_number.x,
                      symbol.part_number.y))

    if filename:
        d.save_svg(f"{filename}.svg")
    return d.as_svg()
